# $Id: ModAreaCode.pm,v 1.1 2010-08-19 19:00:22 uqbar Exp $
# Copyright (c) 2002-2004 Phil Tom <phil@asdf.com>
#
# ModAreaCode.pm - Area Code --> Area mapping
#
# Author: uqbar
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModAreaCode;

@ModAreaCode::ISA = qw(OmniMod);

use strict;

use URI::Escape;
use HTTP::Cookies;

my $on = 1;
my $ua = new LWP::UserAgent;
$ua->agent('Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:0.9.2) Gecko/20010726 Netscape6/6.1');
$ua->timeout(5); # 5s timeout

my $cookie_jar = HTTP::Cookies->new();
$ua->cookie_jar($cookie_jar);

sub handleMessage {
  my $this = shift;
  my $irc = shift;
  my $channel = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  my $output = &get_area_code_info( $message );
  $irc->privmsg( $channel, $output );
	&ModChatLog::saveMessage('omnicron', $output, $channel )
    if defined $output and $output !~ /^\s*$/si;
}

sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  $irc->notice( $nick, &get_area_code_info( $message ) );
}

sub applicable {
  my $message = shift;
  if ( $message =~ /^\s*area\s+/ ) {
    return 1;
  }
  return 0;
}

sub get_area_code_info {
	my $message = shift;
  	my $arg;
  	my $pos = 0;
  	my $ac;
  	if ( $message =~ /^\s*area\s*(.*?)\s*$/si ) {
    	   $ac = $1;
           if( $ac =~ m/\d\d\d/i ) {
	      my $html = &getUrl( "http://www.bennetyee.org/ucsd-pages/area.html" );
              if( $html =~ m/name="$ac">.+?center>.+?center>(..).+?<td>   (.+?)[\(<]/i )
              {
                 return  "Area Code $ac belongs to: $2 (UTC $1)\n";
              }
              else
              {
                 return "No luck finding area code $ac\n";
              }
           } else {
              return "Is $ac really an area code?\n";
           }
        } else {
              return "Cannot parse '$message'\n";
        }
}

sub name() {
  my $this = shift;
  return "area";
}

sub help {
    if ( defined wantarray ) { # do we have help?
        return 1;
    }

    my ($thisi,$irc,$nick,$message) = @_;

    $irc->notice( $nick, "area <3-digit area code> -> looks up city and state information." );
}

sub getUrl {
  my $url = shift;
  my $request = HTTP::Request->new(GET => $url);
  my $response = $ua->request($request);
  my $html = $response->as_string();
  return $html;
}


1;
