# $Id$
# Copyright (c) 2012 Phil Tom <philtom@gmail.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModD3Status;

@ModD3Status::ISA = qw(OmniMod);

use strict;

use Encode;
use LWP::UserAgent;
use HTML::TreeBuilder::XPath;

my $on = 1;
my $ua = LWP::UserAgent->new;
$ua->agent('Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:0.9.2) Gecko/20010726 Netscape6/6.1');
$ua->timeout(5); # 5s timeout

sub help {
  if ( defined wantarray ) { # do we have help?
      return 1;
  }

  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  $irc->notice( $nick, "d3 for Diablo 3 Americas server status");
}

sub name() {
  my $this = shift;
  return "d3";
}

sub handleMessage {
  my $this = shift;
  my $irc = shift;
  my $channel = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );
  if ( $message =~ /^\s*d3\s*$/ ) {
    $irc->privmsg( $channel, &get_d3_status );
  }
}

sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  if ( $message =~ /^\s*d3\s*$/ ) {
    $irc->notice( $nick, &get_d3_status );
  }
}

sub applicable {
  my $message = shift;
  if ( $message =~ /^\s*d3\s*$/ ) {
    return 1;
  } 
  return 0;
}

sub get_d3_status {
  my $url = "http://us.battle.net/d3/en/status";
  my $request = HTTP::Request->new(GET => $url);
  my $response = $ua->request($request);
  my $html = decode_utf8($response->as_string());

  my $tree = HTML::TreeBuilder::XPath->new;
  $tree->parse($html);
  my $retval = "Game Servers: ";
  for my $node ($tree->findnodes('/html/body/div/div[2]/div/div/div[2]/div/div/div/div')) {
    my $region = $node->findvalue('./h3');
    for my $statusNode ( $node->findnodes('./div[1]/div/div[1]') ) {
      my $class = $statusNode->attr('class');
      if ($region && $class =~ /^status-icon (\w+)$/) {
        my $status = $1;
        $retval .= "$region $status, ";
      }
    }
  }
  $tree->delete;

  $retval =~ s/, $//;
  return $retval;
}


1;
