# $Id: ModDNet.pm,v 1.10 2004-05-16 23:27:26 uhtu Exp $
# Copyright (c) 2002-2004 Robert Hagemann <bobzilla@gothictech.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModDNet;

@ModDNet::ISA = qw(OmniMod);

use strict;

use URI::Escape;

my $on = 1;
my $ua = LWP::UserAgent->new;
$ua->agent('Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:0.9.2) Gecko/20010726 Netscape6/6.1');
$ua->timeout(5); # 5s timeout

sub handleMessage {
  my $this = shift;
  my $irc = shift;
  my $channel = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  $irc->privmsg( $channel, &get_dnet_stats( split /\s+/, $message ) );
}

sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  $irc->notice( $nick, &get_dnet_stats( split /\s+/, $message ) );
}

sub applicable {
  my $message = shift;
  if ( $message =~ /^dnet\s+\S+/ ) {
    return 1;
  }
  return 0;
}

sub get_dnet_stats {
  my @words = @_;
  my $url;
  my $request;
  my $response;
  my $html;
  my $script;
  my $keyword;
  my $startpos = 0;
  my $endpos = 0;
  my $rate;
  my $rank;
	my $overallrate;
	my $yesterrate;

  if ($words[1] =~ /mimezine/) {
    $url = 'http://stats.distributed.net/team/tmsummary.php?project_id=8&team=865184013'; 
  }
  else {
    $url = "http://stats.distributed.net/participant/psearch.php?project_id=8\&st=${words[1]}";
  }

  $request = HTTP::Request->new(GET => $url);
  $response = $ua->request($request);
  $html = $response->as_string();
  
	$html =~ s/alt="\+">/>+/sgi;
	$html =~ s/alt="\-">/>-/sgi;
	$html =~ s/<.*?>/ /sgi;

	#print ":$html:\n";

	if ( $html =~ m#Keys/sec:\s+(.+?)\s+(.+?)\s+#si ) {
  	$overallrate = $1 . " Keys/sec";
  	$yesterrate = $2 . " Keys/sec";
	}

	my ($overallrank, $overalldelta, $overallblocks);
	my ($yesterrank, $yesterdelta, $yesterblocks);
	if ( $html =~ /Rank:\s+(\d+)\s+\(\s*([\-\+]\d+)\)\s+(\d+)\s+\(\s*([\-\+]\d+)\)\s+/ ) {
		$overallrank = $1;
		$overalldelta = $2;
		$yesterrank = $3;
		$yesterdelta = $4;
	}
	if ( $html =~ /Blocks:\s+([\d,]+)\s+([\d,]+)\s+/ ) {
		$overallblocks = $1;
		$yesterblocks = $2;
	}
	
  return "$words[1]  Overall: $overallrank ($overalldelta) at $overallrate $overallblocks blocks; Yesterday: $yesterrank ($yesterdelta) at $yesterrate $yesterblocks blocks";
}

1;
