#$Id: ModDropBox.pm,v 1.3 2004-05-15 03:24:50 uhtu Exp $

# Copyright (c) 2004 Hugh Kennedy <kennedyh@engin.umich.edu>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModDropBox;

# unreliable message drop-box module for omnicron. "tell uhtu i said stfu!"
# omnicron will /msg uhtu "you said: 'stfu' on <date/time>"
# the next time uhtu joins any channel omnicron is on.
# only sends the message once. only one message per sender. last message set, wins.
#

@ModDropBox::ISA = qw(OmniMod);

# use actual instance variables
sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self  = $class->SUPER::new();

  # flaky drop box. one message per sender per nick.  nick => {sender=>[when, message]}, 
  $self->{REMINDERS} = {};
  
  bless ($self, $class);          # reconsecrate
  return $self;
}


sub name() {
  my $this = shift;
  return "dropbox";
}

sub help {
  if ( defined wantarray ) { # do we have help?
    return 1;
  }

  my ($this,$irc,$nick,$message) = @_;

  $irc->notice( $nick, "[tell <user> [i said] <message> ] -> send <message> to <user> when they join up next" );

}


sub handlePrivateMessage {
  my ($this, $irc, $nick, $message, $event, $conn ) = @_;
  if ( $message =~ /tell (\w+) (?:i said )?(.+)/i ) {
     #warn "got '$2' from $1";
     $this->{REMINDERS}->{$1}->{$nick} = [time,$2]; 
     $irc->notice( $nick, "okay, $nick." );
  }
}

sub handleJoin {
  my ($this,$irc,$channel,$nick,$parttime,$quittime) = @_;
  if ( $this->{REMINDERS}{$nick} ) {
     #warn "hit for $nick";
     my $reminders = $this->{REMINDERS}{$nick};
     undef $this->{REMINDERS}{$nick};
     for my $sender (keys %{$reminders}) {
#	warn "sender:$sender";
        my ($when,$message) = @{$reminders->{$sender}};
	my $time = ModChatLog::getTimeWithZone($sender,$when,1);
        $irc->notice( $nick, "$sender said to tell you: '$message' on $time" );
     }
  }
}

# return a true value:
1;
