# $Id: ModGoogle.pm,v 1.14 2010-05-10 16:33:54 uhtu Exp $
# Copyright (c) 2002-2004 Phil Tom <phil@asdf.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModGoogle;

@ModGoogle::ISA = qw(OmniMod);

use strict;

use URI::Escape;

my $on = 1;
my $ua = LWP::UserAgent->new;
$ua->agent('Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:0.9.2) Gecko/20010726 Netscape6/6.1');
$ua->timeout(5); # 5s timeout
my $MAX_RESULT_LEN = 200;

sub handleMessage {
  my $this = shift;
  my $irc = shift;
  my $channel = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  $irc->privmsg( $channel, &do_google( split /\s+/, $message ) );
}

sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  $irc->notice( $nick, &do_google( split /\s+/, $message ) );
}

sub applicable {
  my $message = shift;
  if ( $message =~ /^google\s+/ ) {
    return 1;
  }
  return 0;
}

# google search
sub do_google {
  my $url;
  my $keywords = '';
  my @words = @_;
 
  splice @words, 0, 1;
  foreach my $word (@words) {
    $keywords = $keywords . $word . '+';
  }
  chop $keywords;
  my $esckeys = uri_escape($keywords);
  pop @{$ua->requests_redirectable};
  pop @{$ua->requests_redirectable};
  my $urlstring = "http://www.google.com/search?hl=en&source=hp&q=${esckeys}&btnI=I'm+Feeling+Lucky";
  # print "urlstring: $urlstring\n";
  my $request = HTTP::Request->new(GET => $urlstring);
  my $response = $ua->request($request);
  my $html = $response->as_string();
  # print "html: '$html'\n";
  if ($html =~ /^HTTP\/1\.\d+ 200 OK/si) {
    if ( $html =~ /of about.*?<a href="(.*?)">/si ) {
      $url = $1;
      if ( $url =~ m@^/url\?q=(http:.+?)&amp;@ ) {
         $url = $1;
      } 
    } elsif ( $html =~ /Search Results.*?<a href="(.*?)">/si ) {
      $url = $1;
    } else {
      $url = "no results";
    }
  } elsif ( $html =~ /Location: (.*?)\n/si ) {
    $url = $1;
  }
  $url = "no results" unless $url; 
  if (length $url > $MAX_RESULT_LEN ) {
    $url = "bad results for search";
  }

  return $url;
}

1;
