# $Id: ModLists.pm,v 1.4 2004-11-09 23:34:44 bobzilla Exp $
# Copyright (c) 2004 Rachel Engel <rje@nerp.net>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModLists;

@ModLists::ISA = qw(OmniMod);

use strict;

my $on = 1;
my %lists;

open FILE, "<lists";

sub handleMessage {
  return if ! $on;

  my $this = shift;
  my $irc = shift;
  my $channel = shift;
  my $nick = shift;
  my $message = shift;

}

sub name() {
  my $this = shift;
  return "list";
}       

sub add_entry {
     my $message = $_[0];
     my $irc = $_[1];
     my $nick = $_[2];

     if ($message =~ /\s*list\s+(\S+)\s+-l\s+(.*)\s+-e\s+(.+)$/) {
        my $list = $2;
        my $entry = $3; 
        $lists{$list} .= "\0777${entry}";
        $irc->notice( $nick, "($list) += $entry" );
     }
}

sub help {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;

  if ( defined wantarray ) { # do we have help?
    return 1;
  }

  $irc->notice( $nick, "[list add -l <list> -e <entry>] -> Add an entry to a list");
  $irc->notice( $nick, "[list delete -l <list> -n <entry #>] -> Delete an entry from a list");
  $irc->notice( $nick, "[list print -l <list>] -> Print the contents of a list");
  $irc->notice( $nick, "[list lists] -> list all lists");

}

sub delete_entry {
     my $message = $_[0];

     $message =~ /\s*list\s+(\S+)\s+-l\s+(.+)\s+-n\s+(.*)$/;
     my $list = $2;
     my $num = $3;
     my $ret = "";
     my $ii = 0;
     my $first_time = 0;  # no time like it

     my @res = split (/\0777/, $lists{$list});
     for my $res (@res) {
        if (($ii - 1) != $3) {
           if ($first_time++ != 0) {
              $ret .= "\0777";
           }
           $ret .= "${res}";
        }
        $ii++;
     }
     $lists{$list} = $ret;
}

sub print_list {
     my $irc = $_[0];
     my $nick = $_[1];
     my $message = $_[2];
     
     $message =~ /\s*list\s+(\S+)\s+-l\s+(.*)$/;
     my $list = $2;
     my $first_time = 0;  # there goes the white wedding
     my @res = split (/\0777/, $lists{$list});
     my $ii = 0; 

     if (!($lists{$list} =~ /\S+/)) {
        $irc->notice( $nick, " 0");
        return;
     } 
     foreach my $res (@res) {
        if ($first_time++ != 0) {
           $irc->notice( $nick, "$list\[$ii\] = $res\n" );
           $ii++;
        }
     }
}

sub handlePrivateMessage {
  return if ! $on;

  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  my @args = split /\s+/, $message;

  if ($args[0] ne "list") {
     return;
  }

  open_lists();

  if ($args[1] eq "add") {
     add_entry($message, $irc, $nick);
  } elsif ($args[1] eq "delete") {
     delete_entry($message);
  } elsif ($args[1] eq "print") {
     print_list($irc, $nick, $message);
  } elsif ($args[1] eq "lists") {
     foreach my $key (keys %lists) {
        $irc->notice( $nick, "$key" );
     }
  } 

  store_lists();
}

sub open_lists {
   open LIST, "<lists";

   my $data = <LIST>;

   my @arr = split ( /\0775/, $data ); 
   my @arr2;
   my $key;
   my $entry;

   foreach my $matey (@arr) {
      @arr2 = split ( /\0776/, $matey ); 
      $key = $arr2[0];
      $entry = $arr2[1]; 

      $lists{$key} = $entry;
   }

   close LIST;
}     

sub store_lists {
   open LIST, ">lists";

   foreach my $key (keys %lists) {
      print LIST "$key\0776" . $lists{$key} . "\0775";
   }

   close LIST;
}        

1;
