#$Id: ModMendeleev.pm,v 1.1 2008-10-09 17:47:18 uhtu Exp $

# Copyright (c) 2008 Hugh Kennedy <kennedyh@umich.edu>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModMendeleev;

# ModMendeleev is the omnicron periodic table lookup doing forward and reverse
# element code lookups.
# you ask it like:
#  periodic symbol 
# or
#  periodic name
# and it will tell you what it knows:
#  periodic H
#     -> Hydrogen
#  periodic hydrogen
#     -> H
#  
# should have an add interface, but elements change infrequently
#

@ModMendeleev::ISA = qw(OmniMod);

use strict;

# XXX: should re-examine sanity criteria
my $SANITY = 5;

sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self  = $class->SUPER::new();

  while(defined($_=<DATA>)){
    my ($symbol, $name,) = (m/^(\w+)\s+(\w+)\s*$/);
    $self->{name}->{$name} = $symbol;
    $self->{code}->{$symbol}= $name;
  }

  bless ($self, $class);          # reconsecrate
  return $self;
}

sub help {
  if ( defined wantarray ) { # do we have help?
      return 1;
  }

  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  $irc->notice( $nick, "[periodic <symbol or name>]  -> lookup the element" );
}

sub name() {
  my $this = shift;
  return "mendeleev";
}

sub handlePrivateMessage {
  my ($this, $irc, $nick, $message, $event, $conn ) = @_;
  return handleMessage($this, $irc, $nick, $nick, $message);
}

sub handleMessage {
  my ($this, $irc, $channel, $nick, $message ) = @_;

  if ( $message =~ /^periodic\s+(\w+)/i ) {
    my $which = $1;

    if ( $which =~ m/^\w{1,3}$/ && exists $this->{code}->{$which} ) {
      $irc->privmsg( $channel, "$which -> $this->{code}->{$which}" );
    } elsif (defined $which) { 
      my @list;
      for ( keys %{$this->{name}} ) {
        push @list, "$this->{name}->{$_}: $_" if m/$which/i;
      }

      # sanity?
      if ( @list > $SANITY ) {
        $irc->privmsg( $channel, "got too many hits for $which" );
      } elsif ( ! @list ) {
        $irc->privmsg( $channel, "no matches" );
      } else {
        $irc->privmsg( $channel, "$which could be " . ( join ", or ", @list ) );
      }
    } else {
      $irc->privmsg( $channel, "no matches" );
    }
  }
}

'mimezine fecit';

# updates adapated from 
# http://en.wikipedia.org/wiki/Periodic_Table
# and
# http://en.wikipedia.org/wiki/List_of_elements_by_symbol
# 
# Symbol       Name                              
#----------------------------------------------------------------------
__DATA__
Ac 	Actinium
Ag 	Silver
Al 	Aluminium
Am 	Americium
Ar 	Argon
As 	Arsenic
At 	Astatine
Au 	Gold
B 	Boron
Ba 	Barium
Be 	Beryllium
Bh 	Bohrium
Bi 	Bismuth
Bk 	Berkelium
Br 	Bromine
C 	Carbon 
Ca 	Calcium
Cd 	Cadmium
Ce 	Cerium
Cf 	Californium
Cl 	Chlorine
Cm 	Curium
Co 	Cobalt
Cr 	Chromium
Cs 	Caesium
Cu 	Copper
Db 	Dubnium
Ds 	Darmstadtium
Dy 	Dysprosium
Er 	Erbium
Es 	Einsteinium
Eu 	Europium
F 	Fluorine
Fe 	Iron
Fm 	Fermium
Fr 	Francium
Ga 	Gallium
Gd 	Gadolinium
Ge 	Germanium
H 	Hydrogen
He 	Helium
Hf 	Hafnium
Hg 	Mercury
Ho 	Holmium
Hs 	Hassium
I 	Iodine
In 	Indium
Ir 	Iridium
K 	Potassium
Kr 	Krypton
La 	Lanthanum
Li 	Lithium
Lr 	Lawrencium
Lu 	Lutetium
Md 	Mendelevium
Mg 	Magnesium
Mn 	Manganese
Mo 	Molybdenum
Mt 	Meitnerium
N 	Nitrogen
Na 	Sodium
Nb 	Niobium
Nd 	Neodymium
Ne 	Neon
Ni 	Nickel
No 	Nobelium
Np 	Neptunium
O 	Oxygen
Os 	Osmium
P 	Phosphorus
Pa 	Protactinium
Pb 	Lead
Pd 	Palladium
Pm 	Promethium
Po 	Polonium
Pr 	Praseodymium
Pt 	Platinum
Pu 	Plutonium
Ra 	Radium
Rb 	Rubidium
Re 	Rhenium
Rf 	Rutherfordium
Rg 	Roentgenium
Rh 	Rhodium
Rn 	Radon
Ru 	Ruthenium
S 	Sulfur
Sb 	Antimony
Sc 	Scandium
Se 	Selenium
Sg 	Seaborgium
Si 	Silicon
Sm 	Samarium
Sn 	Tin
Sr 	Strontium
Ta 	Tantalum
Tb 	Terbium
Tc 	Technetium
Te 	Tellurium
Th 	Thorium
Ti 	Titanium
Tl 	Thallium
Tm 	Thulium
U 	Uranium
Uub 	Ununbium
Uuh 	Ununhexium
Uuo 	Ununoctium
Uup 	Ununpentium
Uuq 	Ununquadium
Uut 	Ununtrium
V 	Vanadium
W 	Tungsten
Xe 	Xenon
Y 	Yttrium
Yb 	Ytterbium
Zn 	Zinc
Zr 	Zirconium
