# $Id: ModTime.pm,v 1.3 2010-08-25 21:16:23 uqbar Exp $
# Copyright (c) 2002-2004 Phil Tom <phil@asdf.com>
#
# ModTime.pm - Local time in city lookup
# Author: uqbar
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModTime;

@ModTime::ISA = qw(OmniMod);

use strict;

use URI::Escape;
use HTTP::Cookies;

my $on = 1;
my $ua = new LWP::UserAgent;
$ua->agent('Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:0.9.2) Gecko/20010726 Netscape6/6.1');
$ua->timeout(5); # 5s timeout

my $cookie_jar = HTTP::Cookies->new();
$ua->cookie_jar($cookie_jar);

sub handleMessage {
  my $this = shift;
  my $irc = shift;
  my $channel = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  my $output = &get_time( $message );
  $irc->privmsg( $channel, $output );
	&ModChatLog::saveMessage('omnicron', $output, $channel )
    if defined $output and $output !~ /^\s*$/si;
}

sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  $irc->notice( $nick, &get_time( $message ) );
}

sub applicable {
  my $message = shift;
  if ( $message =~ /^\s*timein\s+/ ) {
    return 1;
  }
  return 0;
}

sub get_time {
	my $message = shift;
  	my $arg;
  	my $pos = 0;
  	my $c;
# /^weather\s+.{1,40}$/
  	if ( $message =~ /^\s*timein\s*([a-zA-Z\s\-\.]+)\s*$/si ) {
    	  $c = $1;
  	} else {
    	  return "cannot parse '$message'";
  	}
  
	my $url = "http://www.timeanddate.com/worldclock/results.html?query=$c";
	#return "Using URL $url\n";
	# make request
	my $html = &getUrl( $url );
        my $city;
        my $time;
	if( $html =~ m/<span class=biggest>(.*?)<\/span>/i )
	{
		$city = $1;
		if( $html =~ m/<th class=w5>Current Time<\/th><td><strong id=ct  class=big>(.*?)<\/strong>/i ) 
		{
			$time = $1;
			return "It is $time in $city\n";
		}
	}
	elsif( $html =~ m/<h2 class=\"tophead tc\">Current local times   <\/h2>(.*?)<\/table>/i )
	{
		$html = $1;
		if( $html =~ m/<tr class=c0><td><a href="city.html\?n=(\d{1,7})">/i )
		{
			my $url2 = "http://www.timeanddate.com/worldclock/city.html?n=$1";
			my $html2 = &getUrl( $url2 );
			if( $html2 =~ m/<span class=biggest>(.*?)<\/span>/i)
			{
				$city = $1;
				if( $html2 =~ m/<th class=w5>Current Time<\/th><td><strong id=ct  class=big>(.*?)<\/strong>/i )
				{
					$time = $1;
					return "It is $time in $city\n";
				}
			}

		}
		else
		{
			return "Unable to parse matches.\n";
		}
	}
	else
	{
		return "WTF is '$c'?? Is that a city?\n";
	}
}

sub name() {
  my $this = shift;
  return "timein";
}

sub help {
    if ( defined wantarray ) { # do we have help?
        return 1;
    }

    my ($thisi,$irc,$nick,$message) = @_;

    $irc->notice( $nick, "timein <city> -> looks up current time in the city." );
}

sub getUrl {
  my $url = shift;
  my $request = HTTP::Request->new(GET => $url);
  my $response = $ua->request($request);
  my $html = $response->as_string();
  return $html;
}


1;
