# $Id: ModTranslate.pm,v 1.3 2008-05-12 17:45:41 uhtu Exp $
# Copyright (c) 2003-2004 Robert Hagemann <bobzilla@gothictech.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModTranslate;

@ModTranslate::ISA = qw(OmniMod);

use strict;

use URI::Escape;

my $on = 1;
my $ua = LWP::UserAgent->new;
$ua->agent('Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:0.9.2) Gecko/20010726 Netscape6/6.1');
$ua->timeout(15); # 15s timeout

sub help {
  return if ! $on;

  if ( defined wantarray ) { # do we have help?
    return 1;
  }

  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

	$irc->notice( $nick, "[translate from <language> <text>] translate to english from a language" );
	$irc->notice( $nick, "[translate to <language> <text>] translate from english to a language" );
}

sub name() {
  my $this = shift;
  return "translate";
}

sub handleMessage {
  my $this = shift;
  my $irc = shift;
  my $channel = shift;
  my $nick = shift;
  my $message = shift;
  my $event = shift;

  return if ! $on;

  my $output = &checkAnyMessage( $irc, $nick, $message, $channel, 0 );
	&ModChatLog::saveMessage('omnicron', $output, $channel )
    if defined $output and $output !~ /^\s*$/si;
}

sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;
  my $event = shift;

  return if ! $on;

  &checkAnyMessage( $irc, $nick, $message, $nick, 1 );
}

sub checkAnyMessage {
  my $irc = shift;
  my $nick = shift;
  my $message = shift;
  my $sendTo = shift;
  my $isPrivate = shift;

  my $output;

  if ( $message =~ /^\s*(?:translate|x)\s+(\w+)\s+(\w+)\s+(.*)$/si ) {
    my $type = $1;
    my $msglanguage = $2;
    my $text = $3;

    my $language;
    # non-roman don't work right, pry have to mess with user agent
    if ( $type eq 'from' ) {
      #$language = "zh_en" if $msglanguage =~ /chinese/si;
      #$language = "zt_en" if $msglanguage =~ /chinese\s+trad/si;
      $language = "nl_en" if $msglanguage =~ /dutch/si;
      $language = "fr_en" if $msglanguage =~ /french/si;
      $language = "de_en" if $msglanguage =~ /german/si;
      $language = "el_en" if $msglanguage =~ /greek/si;
      $language = "it_en" if $msglanguage =~ /italian/si;
      #$language = "ja_en" if $msglanguage =~ /japanese/si;
      #$language = "ko_en" if $msglanguage =~ /korean/si;
      $language = "pt_en" if $msglanguage =~ /portuguese/si;
      #$language = "ru_en" if $msglanguage =~ /russian/si;
      $language = "es_en" if $msglanguage =~ /spanish/si;
    }
    elsif ( $type eq 'to' ) {
      #$language = "en_zh" if $msglanguage =~ /chinese/si;
      #$language = "en_zt" if $msglanguage =~ /chinese\s+trad/si;
      $language = "en_nl" if $msglanguage =~ /dutch/si;
      $language = "en_fr" if $msglanguage =~ /french/si;
      $language = "en_de" if $msglanguage =~ /german/si;
      $language = "en_el" if $msglanguage =~ /greek/si;
      $language = "en_it" if $msglanguage =~ /italian/si;
      #$language = "en_ja" if $msglanguage =~ /japanese/si;
      #$language = "en_ko" if $msglanguage =~ /korean/si;
      $language = "en_pt" if $msglanguage =~ /portuguese/si;
      #$language = "en_ru" if $msglanguage =~ /russian/si;
      $language = "en_es" if $msglanguage =~ /spanish/si;
    }
    else {
      $type = undef;
    }

    if ( ! defined $type ) {
      $output = "Bad translation type, use 'from' or 'to'";
    }
    elsif ( ! defined $language ) {
      $output = "Can't translate $type $msglanguage";
    }
    else {
      $text = uri_escape( $text );
      my $urlstring = 
        "http://babelfish.altavista.com/tr?lp=$language\&trtext=$text";
      # print "url: '$urlstring'\n";

      my $request = HTTP::Request->new(GET => $urlstring);
      my $response = $ua->request($request);
      my $transtext = $response->as_string();
      #print "transtext: $transtext\n";
# 	    seeing different results for manual and automated browsing:
# 	    <td class="s" bgcolor="white"><div style="padding: 10px;">chien</div></td>
# 	        <div id="result"><div style="padding:0.6em;">chien</div></div>
#
     if ( $transtext =~ /<div id="result"><div style="padding:0.6em;">(.*?)<\/div>/) { 
        $output = $1;
        print "output: '$output'\n";
      }
    }
  }

  $irc->privmsg( $sendTo, $output ) if defined $output;

  return $output;
}

sub factorial {
  my $value = shift;
  my $output;
  if ( $value > 200 ) {
    $output = "so...big...";
  }
  else {
    $output = 1;
    for ( my $i = 1; $i <= $value; $i++ ) {
      $output *= $i;
    }
  }

  return $output;
}

1;
