# $Id: ModVote.pm,v 1.7 2004-05-16 23:27:26 uhtu Exp $
# Copyright (c) 2003-2004 Robert Hagemann <bobzilla@gothictech.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModVote;

@ModVote::ISA = qw(OmniMod);

use strict;

use Time::Zone;
use POSIX;

my $on = 1;

my $voteTable = "VOTETABLE";

sub help {
  return if ! $on;

  if ( defined wantarray ) { # do we have help?
    return 1;
  }

  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

	$irc->notice( $nick, "[vote create <channel> <question>|<choice1>|<choice2>...]  ->  Create a vote (pvt only)" );
	$irc->notice( $nick, "[vote <voteNumber> <choice>]  ->  Cast your vote (pvt only)" );
	$irc->notice( $nick, "[vote result <voteNumber>]  ->  Get vote results" );
	$irc->notice( $nick, "[vote list [<numberOfVotes>]]  ->  List newest votes (pvt only, default 5)" );
}

sub name() {
  my $this = shift;
  return "vote";
}

sub handleMessage {
  my $this = shift;
  my $irc = shift;
  my $channel = shift;
  my $nick = shift;
  my $message = shift;
  my $event = shift;

  return if ! $on;

  if ( $message =~ /^\s*vote\s+(\d+)\s+(\w)\w*$/si ) {
    #my $response = &castVote( $event, $1, $2 );
    #$irc->notice( $nick, "[vote] " . $response );
  }
  elsif ( $message =~ /^\s*vote\s+result\s+(\d+)\s*$/si ) {
    my $response = &getResult( $1 );
    $irc->privmsg( $channel, "[vote] " . $response );
  }
}

sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;
  my $event = shift;

  return if ! $on;

  if ( $message =~ /^\s*vote\s+create\s+(.*?)\s+(.*?)\s*\|\s*(.*?)\s*$/si ) {
    &createVote( $irc, $nick, $1, $2, $3 );
  }
  elsif ( $message =~ /^\s*vote\s+(\d+)\s+(\w)\w*$/si ) {
    my $response = &castVote( $event, $1, $2 );
    $irc->notice( $nick, "[vote] " . $response );
  }
  elsif ( $message =~ /^\s*vote\s+result\s+(\d+)\s*$/si ) {
    my $response = &getResult( $1 );
    $irc->notice( $nick, "[vote] " . $response );
  }
  elsif ( $message =~ /^\s*vote\s+list\s*(\d+)?\s*$/si ) {
    &getList( $irc, $nick, $1 );
  }
}

sub createVote {
  my ($irc) = shift;
	my ($nick) = shift;
	my ($channel) = shift;
	my ($question) = shift;
	my ($answers) = shift;

  # truncate to 200 chars
  $question =~ s/^(.{200}).*$/$1/sgi;
  # make sure the question is a question
  $question .= "?" if $question !~ /\?\s*$/;

  my $postmsg = "";
  my @answers = split /\|/, $answers;

	my $voteNumData = OmniHash::getHash( $voteTable );

  my $voteNumber = $$voteNumData{ 'votenumber' };
  $voteNumber = -1 if ( ! defined $voteNumber );
  $voteNumber++;
  $$voteNumData{ 'votenumber' } = $voteNumber;

	OmniHash::closeAllHashes();

	my $votedata = OmniHash::getHash( $voteTable . "-" . $voteNumber );

  $$votedata{ 'vote-question' } = $question;
  $$votedata{ 'vote-channel' } = $channel;
  $$votedata{ 'vote-nick' } = $nick;

  # normalize answers
  my $letter = "A";
  foreach my $answer ( @answers ) {
    $answer =~ s/^\s*(.*?)\s*$/$1/sgi;
    # protect against evil answers
    $answer =~ s/\((.)\)/$1/sgi;
    $answer =~ s/\<(\d+)\>/$1/sgi;
    # truncate to 100 chars
    $answer =~ s/^(.{100}).*$/$1/sgi;
    $postmsg .= "($letter) $answer ";
    
    $$votedata{ 'vote-answer-' . $letter } = $answer;

    $letter++;
  }

	OmniHash::closeAllHashes();

  # done out of order cuz we need voteNumber
  $postmsg = "New Vote from $nick! ($voteNumber)  $question -- " . $postmsg;

  # send notice of fun new vote
  $irc->privmsg( $channel, $postmsg );

}

sub castVote {
  my $event = shift;
  my $voteNumber = shift;
  my $choice = shift;

  $choice = uc $choice;

  return "ERROR: choice $choice does not exist"
    if $choice !~ /^\w$/;

	my $votedata = OmniHash::getHash( $voteTable . "-" . $voteNumber );
  return "ERROR: vote number $voteNumber does not exist" 
    if ! defined $$votedata{ 'vote-question' };
  my $voteanswer = $$votedata{ 'vote-answer-' . $choice };
  return "ERROR: choice $choice does not exist"
    if ! defined $voteanswer or $voteanswer =~ /^\s*$/;

  $$votedata{ 'vote-choice-' . $event->user . '-' . $event->host } = $choice;
  
	OmniHash::closeAllHashes();

  return "Vote for $choice on vote $voteNumber was successful (" 
    . $voteanswer . ")";
}

sub getResult {
  my $voteNumber = shift;

	my $votedata = OmniHash::getHash( $voteTable . "-" . $voteNumber );

  my $question = $$votedata{ 'vote-question' };
  return "ERROR: vote number $voteNumber does not exist" 
    if ! defined $question;

  # collect votes
  my %votes = ();
  foreach my $key ( keys %{$votedata} ) {
    next if $key !~ /^vote-choice-/;

    my $votechoice = $$votedata{ $key };
    $votes{ $votechoice } = 0 if ! defined $votes{ $votechoice };
    $votes{ $votechoice }++;
  }

  # start making response
  my $postmsg = "Vote Results ($voteNumber): $question -- ";

  my $letter = "A";
  while ( defined $$votedata{ 'vote-answer-' . $letter } ) {
    $votes{ $letter } = 0 if ! defined $votes{ $letter };

    my $answer = $$votedata{ 'vote-answer-' . $letter };
    $postmsg .= "($letter) $answer <" . $votes{ $letter } . "> ";
    $letter++;
  }
	
  OmniHash::closeAllHashes();

  return $postmsg;
}

sub getList {
  my $irc = shift;
  my $nick = shift;
  my $number = shift;

  $number = 5 if ! defined $number;

	my $voteNumData = OmniHash::getHash( $voteTable );

  my $voteNumber = $$voteNumData{ 'votenumber' };
  $voteNumber = -1 if ( ! defined $voteNumber );

	OmniHash::closeAllHashes();

  for ( my $i = $voteNumber; $number > 0 and $i >= 0; $number--, $i-- ) {
    my $response = &getResult( $i );
    $irc->notice( $nick, $response );
  }
}

1;
