# $Id: ModWiGLE.pm,v 1.28 2011-02-12 19:54:00 bobzilla Exp $
# Copyright (c) 2002-2004 Robert Hagemann <bobzilla@gothictech.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModWiGLE;

@ModWiGLE::ISA = qw(OmniMod);

use strict;

use URI::Escape;

my $on = 1;
my $ua = LWP::UserAgent->new;
$ua->agent('Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:0.9.2) Gecko/20010726 Netscape6/6.1');
$ua->timeout(5); # 5s timeout

sub handleMessage {
  my $this = shift;
  my $irc = shift;
  my $channel = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );
  if ( $message =~ /^wigle\s*$/ ) {
    $irc->privmsg( $channel, &get_wigle_stats( split /\s+/, $message ) );
  } elsif ( $message =~ /^wwwd3?\s*$/ ) {
    $irc->privmsg( $channel, &get_wigle_wwwd_stats( split /\s+/, $message ) );
  } elsif ( $message =~ /^wiglevs\s+/ ) {
    $irc->privmsg( $channel, &get_wigle_versus( split /\s+/, $message ) );
  }
}

sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  if ( $message =~ /^wigle\s*$/ ) {
    $irc->notice( $nick, &get_wigle_stats( split /\s+/, $message ) );
  } elsif ( $message =~ /^wwwd3?\s*$/ ) {
    $irc->notice( $nick, &get_wigle_wwwd_stats( split /\s+/, $message ) );
  } elsif ( $message =~ /^wiglevs\s+/ ) {
    $irc->notice( $nick, &get_wigle_versus( split /\s+/, $message ) );
  }
}

sub applicable {
  my $message = shift;
  if ( $message =~ /^wigle\s*$/ ) {
    return 1;
  } elsif ( $message =~ /^wwwd3?\s*$/ ) {
    return 1;
  } elsif ( $message =~ /^wiglevs\s+/ ) {
    return 1;
  }
  return 0;
}

sub get_wigle_stats {
  my @words = @_;

	my $url = 'http://wigle.net/';
  my $request = HTTP::Request->new(GET => $url);
  my $response = $ua->request($request);
  my $html = $response->as_string();
	
	my $retval = "No results";
	if ( $html =~ /"stats">\s*([\d,]+?)\s*<\/span>\s*wifi/si ) {
		my $num = $1;
		$html =~ /"stats">\s*([\d,]+?)\s*<\/span>\s*cellular/si;
		my $cell = $1;
		$html =~ /"stats">\s*([\d,]+?)\s*<\/span>\s*unique observations/si;
		my $loc = $1;
		$retval = "There are $num wifi and $cell cellular networks on WiGLE.net, from $loc observations.";
	}

	return $retval;
}

sub get_wigle_wwwd_stats {
  my $url = 'http://wigle.net/gps/gps/GPSDB/stats/';
  my $request = HTTP::Request->new(GET => $url);
  my $response = $ua->request($request);
  my $html = $response->as_string();

  my $retval = "No results";
  if ( $html =~ m@New unique networks during <a href="http://www.worldwidewardrive.org/">wwwd3</a>: ([\d,]+)@si ) {
    my $num = $1;
    $retval = "There are $num new wireless networks from wwwd3 on WiGLE.net";
  }

  return $retval;
}

sub get_wigle_versus {
  my @words = @_;
  my $resultStr = '';
  my $maxCount = 0;
  my $maxName = '';

  my $tie = 0;

  splice @words, 0, 1;
  foreach my $word (@words) {
	my $baseurl = 'http://wigle.net/gps/gps/statsAPISupport/countforname?name=';
    my $request = HTTP::Request->new(GET => $baseurl.$word);
    my $response = $ua->request($request);
    if ($response->is_success) {

      if ( $response->content =~ /\d+/si ) {
	  my $res = $response->content;
	
    	  $resultStr .= "($word: $res) ";

    	  if ($res > $maxCount) {
	    $maxCount = $res ;
	    $maxName = $word;
	    $tie = 0;
    	  } elsif ($res > $maxCount) {
	    $maxName .= " and $word";
	    $tie = 1;
	  }
	} else {
		$resultStr .= "(wigle: request for $word failed) "
	}
      }
    }
	
  if ($resultStr eq "") {
	return "wigle: unable to compare terms";
  }	
	
  if ($tie == 1) {
	$resultStr = "$maxName tie! ".$resultStr; 
  } else {
  	$resultStr = "$maxName wins! ".$resultStr;
  }
  return $resultStr;
}

1;
