# $Id: ModZip.pm,v 1.3 2010-08-20 22:07:28 uqbar Exp $
# Copyright (c) 2002-2004 Phil Tom <phil@asdf.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModZip;

@ModZip::ISA = qw(OmniMod);

use strict;

use URI::Escape;
use HTTP::Cookies;

my $on = 1;
my $ua = new LWP::UserAgent;
$ua->agent('Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:0.9.2) Gecko/20010726 Netscape6/6.1');
$ua->timeout(5); # 5s timeout

my $cookie_jar = HTTP::Cookies->new();
$ua->cookie_jar($cookie_jar);

sub handleMessage {
  my $this = shift;
  my $irc = shift;
  my $channel = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  my $output = &get_zip_info( $message );
  $irc->privmsg( $channel, $output );
	&ModChatLog::saveMessage('omnicron', $output, $channel )
    if defined $output and $output !~ /^\s*$/si;
}

sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  $irc->notice( $nick, &get_zip_info( $message ) );
}

sub applicable {
  my $message = shift;
  if ( $message =~ /^\s*zip\s+/ ) {
    return 1;
  }
  return 0;
}

sub get_zip_info {
	my $message = shift;
  	my $arg;
  	my $pos = 0;
  	my $zc;
	if ( $message =~ /^\s*zip\s*(.*?)\s*$/si ) {
	  	#if ( $message =~ /^\s*zip\s*([\^\&\.\:\-\w]+)\s*$/si ) {
    	$zc = $1;
  	} else {
    	return "cannot parse '$message'";
  	}
  
	if( $zc =~ m/^\d{5}$/ )
	{
		# print "Zip Code: $zc";
		my $url = "http://zipinfo.com/cgi-local/zipsrch.exe?ac=ac&tz=tz&zip=$zc&Go=Go";
		#print "Using URL $url\n";
		# make request
		my $html = &getUrl( $url );

		if( $html =~ m/does not deliver/ ) {
			return "The USPS does not deliver to zip code $zc\n";
		} elsif( $html =~ m/not currently assigned/ ) {
			return "$zc is not currently assigned.\n";
		} elsif( $html =~ m/<th>Mailing<BR>Name<\/th><th>State<BR>Code<\/th><th>ZIP<BR>Code<\/th><th>Area<BR>Code<\/th><th>Time<BR>Zone<\/th><th>Daylight<BR>Time<\/th><\/tr><tr><td align=center>(.*?)<\/font><\/td><td align=center>(.*?)<\/font><\/td><td align=center>(.*?)<\/font><\/td><td align=center>(.*?)<\/font><\/td><td align=center>(.*?)<\/font><\/td><td align=center>(.*?)<\/font><\/td><\/tr><\/table>/i ) {
		#		$city = $1;
		#		$state = $2;
			return $3." --> $1, $2 (Area Code: $4 | Time Zone: $5 | Daylight Savings: $6)\n";
		} else {
			return "No match found for $zc\n";
		}
	}
	else
	{
		my @inps = split(',', $zc);
		if( $inps[1] =~ m/[a-z]{2}$/i )
		{
			# Save state and format for URL
			my $cityurl = $inps[0];
			my $city = $inps[0];
			$cityurl =~ s/ /\+/g;

			# To uppercase and strip spaces
			my $state = uc($inps[1]);
			$state =~ s/\s//g;

			my $url = "http://www.zip-codes.com/search.asp?fld-city=$cityurl&fld-state=$state&selectTab=4&Submit=Search+for+ZIP+Code";

			my $html = &getUrl( $url );
			$html =~ tr/\r\n//d;

			if( $html =~ m/No records match your search parameters/ ) {
				return "Could not locate $city, $state\n";
			} elsif( $html =~ m/<th>.+?(\d{5}).+?area-code-(\d{3})\.asp.+?county\/\D\D-(.+?)\.asp.*US<\/td><\/tr><\/table>/i ) {
			#	$1 - Zip Code
			#	$2 - First Area Code
			#	$3 - County
				return "".uc($city).", $state is in $3 county. It has a zip code of $1 and a primary area code of $2.\n";
			} else {
				return "Could not locate ".uc($city).", ".uc($state)."\n$url\n";
			}
		}
		else
		{
			return "Could not locate '$zc'. Need <city, state-code>.";
		}
	}
}

sub name() {
  my $this = shift;
  return "zip";
}

sub help {
    if ( defined wantarray ) { # do we have help?
        return 1;
    }

    my ($thisi,$irc,$nick,$message) = @_;

    $irc->notice( $nick, "zip <5-digit-zip> -> looks up city and state information, and also area code, for a zip code." );
}

sub getUrl {
  my $url = shift;
  my $request = HTTP::Request->new(GET => $url);
  my $response = $ua->request($request);
  my $html = $response->as_string();
  return $html;
}


1;
